import intervals.SimpleInterval;

import java.util.Random;

import calendar.Calendar;
import calendar.CalendarEntry;
import calendar.CalendarListener;
import calendar.TimeFrameCalendarView;

public class Test {
    public static void main(String[] args) {
        final Calendar cal = new Calendar ();
        
        cal.add ( new CalendarEntry ( 5, 10, "A" ) );
        cal.add ( new CalendarEntry ( 2, 7, "B" ) );
        cal.add ( new CalendarEntry ( 10, 20, "C" ) );
        cal.add ( new CalendarEntry ( 25, 30, "D" ) );

        System.out.println ( "Adding noise ..." );
        
        final Random r = new Random ();
        for (int i = 0; i != 20000; ++i) {
            int start;
            do {
                start = r.nextInt () % 30000 - 15000;
            } while ( start < -10000 || start > 10000 );
            int length;
            do {
                length = r.nextInt ( 10 );
            } while ( start + length < start );
            cal.add ( new CalendarEntry ( start, start + length, "noise" ) );            
        }

        System.out.println ();
        print ( cal );

        System.out.println ( "Creating timeframe view for [-10, 10] ..." );

        final TimeFrameCalendarView view =
            new TimeFrameCalendarView ( cal, new SimpleInterval ( -10, 10 ) );
        
        System.out.println ( "Contents:" );
        System.out.println ( view.getEntries () );
        
        view.registerListener ( new PrintListener () );
        
        System.out.println ( "Changing entries ..." );

        for (int i = 0; i != 10000; ++i) {
            final int index = r.nextInt ( cal.getEntries ().size () );
            final CalendarEntry e = cal.getEntries ().at ( index );
            int start;
            do {
                start = r.nextInt () % 30000 - 15000;
            } while ( start < -10000 || start > 10000 );
            int length;
            do {
                length = r.nextInt ( 10 );
            } while ( start + length < start );
            e.moveEntry ( start, start + length );
        }

        System.out.println ();
        print ( cal );
    }

    private static void print(Calendar cal) {
        cal.getCompleteView ().printStatistics ();
        System.out.println ();

//        System.out.println ( "All entries:" );
//        System.out.println ( cal.getEntries () );
        
        for (int i = -30; i < 30; i += 10) {
            System.out.println ( "All entries overlapping [" + i + ", " + (i+9) + "]:" );
            System.out.println ( cal.getCompleteView ()
                                 .getOverlappingEntries ( new SimpleInterval
                                                          ( i, i+9 ) ) );
        }
    }
}

class PrintListener implements CalendarListener {
    public void addedEntry(CalendarEntry e) {
        System.out.println("Added entry: " + e);
    }

    public void removedEntry(CalendarEntry e) {
        System.out.println("Removed entry: " + e);
    }
}
