import intervals.Interval;
import calendar.Calendar;
import calendar.CalendarEntry;
import calendar.CalendarListener;
import calendar.TimeFrameCalendarView;

public class TimeFrameDisplay implements CalendarListener {

    private final Calendar cal;
    private final Interval timeFrame;
    private final TimeFrameCalendarView timeFrameView;
    
    /*@
      @ public invariant cal != null
      @                  && timeFrame != null
      @                  && timeFrameView != null;
      @ public invariant timeFrameView.getListenersNum () == 1
      @                  && timeFrameView.getListener ( 0 ) == this
      @                  && timeFrameView.getTimeFrame () == timeFrame;
      @*/
    
    /*@
      @ public normal_behavior
      @  requires cal != null && timeFrame != null;
      @*/
    public TimeFrameDisplay(Calendar cal, Interval timeFrame) {
        this.cal = cal;
        this.timeFrame = timeFrame;
        this.timeFrameView = new TimeFrameCalendarView ( cal, timeFrame );
        this.timeFrameView.registerListener ( this );
    }

    /*@
      @ public normal_behavior
      @  requires entry != null;
      @  requires overlapping ( timeFrame, entry );
      @  ensures lastEntryAdded == entry;
      @*/
    public void add(CalendarEntry entry) {
        cal.add ( entry );
    }
    
    private CalendarEntry lastEntryAdded = null;
    
    public void addedEntry(CalendarEntry e) {
        lastEntryAdded = e;
    }

    public void removedEntry(CalendarEntry e) {}
    
    public /*@ pure @*/ static boolean overlapping(Interval iv1, Interval iv2) {
        return iv1.getStart () >= iv2.getStart ()
               && iv1.getStart () <= iv2.getEnd ()
               || iv2.getStart () >= iv1.getStart ()
               && iv2.getStart () <= iv1.getEnd ();
    }
}
