package calendar;

import observerPattern.Subject;
import intervals.Interval;

/**
 * Class representing calendar entries. Such entries are intervals (have a start
 * and an end point that are integers) and a description in form of a string.
 * Entries are immutable, it is, for instance, possible to move an entries to
 * different time (change the start and the end point). It is possible to
 * register as observer and be notified about all modifications that occur for
 * an entry.
 */
public class CalendarEntry extends Subject implements Interval {
    
    private int start, end;
    private String description;
    
    /*@
      @ public normal_behavior
      @  requires start <= end;
      @*/
    public CalendarEntry(int start, int end, String description) {
        this.start = start;
        this.end = end;
        this.description = description;
    }

    public /*@ pure @*/ int getStart() {
        return start;
    }
    
    public /*@ pure @*/ int getEnd() {
        return end;
    }

    /*@
      @ public normal_behavior
      @  requires newStart <= newEnd;
      @*/
    public void moveEntry(int newStart, int newEnd) {
        prepareObservers ();
        start = newStart;
        end = newEnd;
        notifyObservers ();
    }
    
    public /*@ pure @*/ String getDescription() {
        return description;
    }

    public String toString() {
        return "[" + getStart () + ", " + getEnd () + "] " + getDescription ();
    }
}
