package calendar;

import intervals.Interval;

public class TimeFrameCalendarView extends CalendarModificationServer
                                   implements CalendarView, CalendarListener {
    
    private final Calendar cal;
    private final SortedCalendarView completeView;
    private final Interval timeFrame;
    
    /*@
      @ private invariant completeView != null;
      @ private invariant timeFrame != null;
      @ private invariant cal != null;
      @ private invariant
      @       (\exists int i; 0 <= i && i < cal.getListenersNum ();
      @         cal.getListener ( i ) == this &&
      @         (\forall int j; 0 <= j && j < cal.getListenersNum () && i != j;
      @                         cal.getListener ( j ) != this));
      @*/
    
	/*@
      @ public normal_behavior
      @  requires cal != null && timeFrame != null;
      @*/
    public TimeFrameCalendarView(Calendar cal, Interval timeFrame) {
        this.cal = cal;
        completeView = cal.getCompleteView ();
        this.timeFrame = timeFrame;
        cal.registerListener ( this );
    }

    public CalendarEntrySeq getEntries() {
        return completeView.getOverlappingEntries ( timeFrame );
    }

    public void addedEntry(CalendarEntry e) {
        if ( overlapping ( timeFrame, e ) ) fireAddedEntry ( e );
    }

    public void removedEntry(CalendarEntry e) {
        if ( overlapping ( timeFrame, e ) ) fireRemovedEntry ( e );
    }
    
    private boolean overlapping(Interval iv1, Interval iv2) {
        return iv1.getStart () >= iv2.getStart ()
               && iv1.getStart () <= iv2.getEnd ()
               || iv2.getStart () >= iv1.getStart ()
               && iv2.getStart () <= iv1.getEnd ();
    }

    public Interval getTimeFrame() {
		return timeFrame;
	}
}
